@@ -15,9 +15,6 @@ import re
 import time
 import threading
 
-import six
-from six.moves import range
-
 from .b2http import ResponseContextManager
 from .exception import (
     BadJson,
@@ -206,7 +203,7 @@ class FileSimulator(object):
             headers['Content-Range'] = 'bytes %d-%d/%d' % (
                 range_[0], range_[0] + content_length, len(self.data_bytes)
             )  # yapf: disable
-        for key, value in six.iteritems(self.file_info):
+        for key, value in self.file_info.items():
             headers['x-bz-info-' + key] = value
         return headers
 
@@ -254,20 +251,18 @@ class FileSimulator(object):
 
     def finish(self, part_sha1_array):
         last_part_number = max(part.part_number for part in self.parts if part is not None)
-        for part_number in six.moves.range(1, last_part_number + 1):
+        for part_number in range(1, last_part_number + 1):
             if self.parts[part_number] is None:
                 raise MissingPart(part_number)
         my_part_sha1_array = [
-            self.parts[part_number].content_sha1
-            for part_number in six.moves.range(1, last_part_number + 1)
+            self.parts[part_number].content_sha1 for part_number in range(1, last_part_number + 1)
         ]
         if part_sha1_array != my_part_sha1_array:
             raise ChecksumMismatch(
                 'sha1', expected=str(part_sha1_array), actual=str(my_part_sha1_array)
             )
-        self.data_bytes = six.b('').join(
-            self.parts[part_number].part_data
-            for part_number in six.moves.range(1, last_part_number + 1)
+        self.data_bytes = b''.join(
+            self.parts[part_number].part_data for part_number in range(1, last_part_number + 1)
         )
         self.content_length = len(self.data_bytes)
         self.action = 'upload'
@@ -307,7 +302,9 @@ class FakeResponse(object):
 
     def iter_content(self, chunk_size=1):
         start = 0
+        rnd = random.Random(self.url)
         while start <= len(self.data_bytes):
+            time.sleep(rnd.random() * 0.01)
             yield self.data_bytes[start:start + chunk_size]
             start += chunk_size
 
@@ -420,7 +417,7 @@ class BucketSimulator(object):
         return self.file_id_to_file[file_id].as_upload_result()
 
     def get_upload_url(self):
-        upload_id = six.next(self.upload_url_counter)
+        upload_id = next(self.upload_url_counter)
         upload_url = 'https://upload.example.com/%s/%s' % (self.bucket_id, upload_id)
         return dict(bucketId=self.bucket_id, uploadUrl=upload_url, authorizationToken=upload_url)
 
@@ -431,8 +428,8 @@ class BucketSimulator(object):
     def hide_file(self, file_name):
         file_id = self._next_file_id()
         file_sim = self.FILE_SIMULATOR_CLASS(
-            self.account_id, self.bucket_id, file_id, 'hide', file_name, None, "none", {},
-            six.b(''), six.next(self.upload_timestamp_counter)
+            self.account_id, self.bucket_id, file_id, 'hide', file_name, None, "none", {}, b'',
+            next(self.upload_timestamp_counter)
         )
         self.file_id_to_file[file_id] = file_sim
         self.file_name_and_id_to_file[file_sim.sort_key()] = file_sim
@@ -470,7 +467,7 @@ class BucketSimulator(object):
         copy_file_sim = self.FILE_SIMULATOR_CLASS(
             self.account_id, destination_bucket_id, new_file_id, 'copy', new_file_name,
             file_sim.content_type, file_sim.content_sha1, file_sim.file_info, data_bytes,
-            six.next(self.upload_timestamp_counter)
+            next(self.upload_timestamp_counter)
         )
 
         if metadata_directive is MetadataDirectiveMode.REPLACE:
@@ -487,7 +484,7 @@ class BucketSimulator(object):
         result_files = []
         next_file_name = None
         prev_file_name = None
-        for key in sorted(six.iterkeys(self.file_name_and_id_to_file)):
+        for key in sorted(self.file_name_and_id_to_file):
             (file_name, file_id) = key
             assert file_id
             if start_file_name <= file_name and file_name != prev_file_name:
@@ -517,7 +514,7 @@ class BucketSimulator(object):
         result_files = []
         next_file_name = None
         next_file_id = None
-        for key in sorted(six.iterkeys(self.file_name_and_id_to_file)):
+        for key in sorted(self.file_name_and_id_to_file):
             (file_name, file_id) = key
             if (start_file_name < file_name) or (
                 start_file_name == file_name and
@@ -541,7 +538,7 @@ class BucketSimulator(object):
         start_file_id = start_file_id or self.FIRST_FILE_ID
         max_file_count = max_file_count or 100
         all_unfinished_ids = set(
-            k for (k, v) in six.iteritems(self.file_id_to_file)
+            k for (k, v) in self.file_id_to_file.items()
             if v.action == 'start' and k <= start_file_id and
             (prefix is None or v.name.startswith(prefix))
         )
@@ -568,7 +565,7 @@ class BucketSimulator(object):
         file_id = self._next_file_id()
         file_sim = self.FILE_SIMULATOR_CLASS(
             self.account_id, self.bucket_id, file_id, 'start', file_name, content_type, 'none',
-            file_info, None, six.next(self.upload_timestamp_counter)
+            file_info, None, next(self.upload_timestamp_counter)
         )  # yapf: disable
         self.file_id_to_file[file_id] = file_sim
         self.file_name_and_id_to_file[file_sim.sort_key()] = file_sim
@@ -600,7 +597,7 @@ class BucketSimulator(object):
         self, upload_id, upload_auth_token, file_name, content_length, content_type, content_sha1,
         file_infos, data_stream
     ):
-        data_bytes = data_stream.read()
+        data_bytes = self._simulate_chunked_post(data_stream, content_length)
         assert len(data_bytes) == content_length
         if content_sha1 == HEX_DIGITS_AT_END:
             content_sha1 = data_bytes[-40:].decode()
@@ -612,7 +609,7 @@ class BucketSimulator(object):
         file_id = self._next_file_id()
         file_sim = self.FILE_SIMULATOR_CLASS(
             self.account_id, self.bucket_id, file_id, 'upload', file_name, content_type,
-            content_sha1, file_infos, data_bytes, six.next(self.upload_timestamp_counter)
+            content_sha1, file_infos, data_bytes, next(self.upload_timestamp_counter)
         )
         self.file_id_to_file[file_id] = file_sim
         self.file_name_and_id_to_file[file_sim.sort_key()] = file_sim
@@ -620,7 +617,7 @@ class BucketSimulator(object):
 
     def upload_part(self, file_id, part_number, content_length, sha1_sum, input_stream):
         file_sim = self.file_id_to_file[file_id]
-        part_data = input_stream.read()
+        part_data = self._simulate_chunked_post(input_stream, content_length)
         assert len(part_data) == content_length
         if sha1_sum == HEX_DIGITS_AT_END:
             sha1_sum = part_data[-40:].decode()
@@ -638,8 +635,37 @@ class BucketSimulator(object):
             contentSha1=sha1_sum
         )  # yapf: disable
 
+    def _simulate_chunked_post(
+        self, stream, content_length, min_chunks=4, max_chunk_size=8096, simulate_retry=True
+    ):
+        chunk_size = max_chunk_size
+        chunks_num = self._chunks_number(content_length, chunk_size)
+        if chunks_num < min_chunks:
+            chunk_size = max(content_length // min_chunks, 1)
+        loop_count = 2 if simulate_retry else 1
+        stream_data = None
+        for _ in range(loop_count):
+            chunks = []
+            stream.seek(0)  # we always do this in `do_post` in `b2http` so we want it here *always*
+            while True:
+                data = stream.read(chunk_size)
+                chunks.append(data)
+                if not data:
+                    break
+            _stream_data = b''.join(chunks)
+            if stream_data is not None:
+                assert _stream_data == stream_data
+            stream_data = _stream_data
+        return stream_data
+
+    def _chunks_number(self, content_length, chunk_size):
+        chunks_number = content_length // chunk_size
+        if content_length % chunk_size > 0:
+            chunks_number = chunks_number + 1
+        return chunks_number
+
     def _next_file_id(self):
-        return str(six.next(self.file_id_counter))
+        return str(next(self.file_id_counter))
 
 
 class RawSimulator(AbstractRawApi):
@@ -793,7 +819,7 @@ class RawSimulator(AbstractRawApi):
         self._assert_account_auth(api_url, account_auth_token, account_id, 'writeBuckets')
         if bucket_name in self.bucket_name_to_bucket:
             raise DuplicateBucketName(bucket_name)
-        bucket_id = 'bucket_' + str(six.next(self.bucket_id_counter))
+        bucket_id = 'bucket_' + str(next(self.bucket_id_counter))
         bucket = self.BUCKET_SIMULATOR_CLASS(
             account_id, bucket_id, bucket_name, bucket_type, bucket_info, cors_rules,
             lifecycle_rules
@@ -1007,8 +1033,7 @@ class RawSimulator(AbstractRawApi):
 
         # Do the query
         sorted_buckets = [
-            self.bucket_name_to_bucket[name]
-            for name in sorted(six.iterkeys(self.bucket_name_to_bucket))
+            self.bucket_name_to_bucket[name] for name in sorted(self.bucket_name_to_bucket)
         ]
         bucket_list = [
             bucket.bucket_dict()
@@ -1017,7 +1042,7 @@ class RawSimulator(AbstractRawApi):
         return dict(buckets=bucket_list)
 
     def _get_bucket_id_or_none_for_bucket_name(self, bucket_name):
-        for bucket in six.itervalues(self.bucket_name_to_bucket):
+        for bucket in self.bucket_name_to_bucket.values():
             if bucket.bucket_name == bucket_name:
                 return bucket.bucket_id
 
